import datetime
import decimal
from typing import Any

NUMBER = float
DECIMAL = decimal.Decimal
NUMERIC = float
INT = int
INTEGER = int
BIGINT = int
SMALLINT = int
TINYINT = int
BYTEINT = int
FLOAT = float
FLOAT4 = float
FLOAT8 = float
DOUBLE = float
REAL = float
VARCHAR = str
CHAR = str
CHARACTER = str
STRING = str
TEXT = str
BINARY = bytes
VARBINARY = bytes
BOOLEAN = bool
DATE = datetime.date
DATETIME = datetime.datetime
TIME = datetime.time
TIMESTAMP = datetime.datetime
TIMESTAMP_LTZ = datetime.datetime
TIMESTAMP_NTZ = datetime.datetime
TIMESTAMP_TZ = datetime.datetime
VARIANT = Any
OBJECT = dict[str, Any]
ARRAY = list[Any]
GEOGRAPHY = Any
GEOMETRY = Any
